import 'package:adeptgraphql/src/resource/subscription_resource_status.dart';

class SubscriptionResource<T> {
  SubscriptionResourceStatus status;
  String? message;
  int? code;
  dynamic exception;
  dynamic extras;
  T? data;

  SubscriptionResource(
      {this.status = SubscriptionResourceStatus.ERROR,
      this.data,
      this.message,
      this.code,
      this.exception,
      this.extras}) {
    status = status;
    data = data;
    message = message;
    code = code;
    exception = exception;
    extras = extras;
  }

  factory SubscriptionResource.loading(T? data, {dynamic extras}) =>
      SubscriptionResource(
          status: SubscriptionResourceStatus.LOADING,
          data: data,
          extras: extras);
  factory SubscriptionResource.data(T? data, {dynamic extras}) =>
      SubscriptionResource(
          status: SubscriptionResourceStatus.DATA, data: data, extras: extras);
  factory SubscriptionResource.error(T? data, String? message,
          {int? code, dynamic exception, dynamic extras}) =>
      SubscriptionResource(
          status: SubscriptionResourceStatus.ERROR,
          data: data,
          message: message,
          code: code,
          exception: exception,
          extras: extras);
  factory SubscriptionResource.done({T? data, dynamic extras}) =>
      SubscriptionResource(
          status: SubscriptionResourceStatus.DONE, data: data, extras: extras);

  bool exists() => data != null;

  bool equals(Object o) {
    if (this == o) return true;

    if (o is! SubscriptionResource) return false;

    SubscriptionResource subscriptionResource = o;
    if (status != subscriptionResource.status) return false;
    if (code != subscriptionResource.code) return false;
    if (message != subscriptionResource.message) return false;
    return data != subscriptionResource.data;
  }

  SubscriptionResource<T> clone(
      {required SubscriptionResource<T> currentData,
      SubscriptionResource<T>? newData,
      bool merge = false}) {
    final clone = SubscriptionResource<T>()
      ..status = newData?.status ?? currentData.status
      ..message = currentData.message
      ..code = newData?.code
      ..exception = newData?.exception ?? currentData.exception
      ..extras = newData?.extras ?? currentData.extras
      ..data = newData?.data ?? currentData.data;

    if (merge) {
      currentData.status = clone.status;
      currentData.message = clone.message;
      currentData.code = clone.code;
      currentData.exception = clone.exception;
      currentData.extras = clone.extras;
      currentData.data = clone.data;
    }

    return clone;
  }

  static bool hasData(SubscriptionResource? subscriptionResource) =>
      subscriptionResource?.exists() ?? false;
}
